@@ -118,7 +118,7 @@ void print_event (struct inotify_event *event)
 	printf ("EVENT ON WD=%d\n", event->wd);
 	print_mask (event->mask);
 	if (event->len)
-		printf ("FILENAME=%s\n", event->filename);
+		printf ("FILENAME=%s\n", event->name);
 	printf("\n");
 }
 
@@ -204,22 +204,27 @@ int read_and_print_events (queue_t q, int fd)
 	return 0;
 }
 
-int watch_dir (int fd, const char *dirname, long int mask)
+int watch_dir (int fd, const char *dirname, __u32 mask)
 {
 	struct inotify_watch_request iwr;
-	iwr.dirname = strdup (dirname);
-	iwr.mask = mask;
 	int wd;
+	int file;
+
+	file = open (dirname, O_RDONLY);
+	if (file < 0) {
+		perror ("open");
+		return -1;
+	}
+
+	iwr.fd = file;
+	iwr.mask = mask;
 
 	wd = ioctl(fd, INOTIFY_WATCH, &iwr);
 
-	if (wd < 0) {
-		printf("ioctl(fd, INOTIFY_WATCH, {%s, %d}) = %d", iwr.dirname, iwr.mask, wd);
-		fflush(stdout);
-		perror (" ");
-	}
+	if (wd < 0)
+		perror ("ioctl");		
 
-	free (iwr.dirname);
+	close (file);
 
 	printf("%s WD=%d\n", dirname, wd);
 	return wd;
